
//==============================================================================
//
//           INFRA_SONY_5CANAIS_C
//
//   AUTOR: CLAUDIO LRIOS
//   DATA: 15/01/2014
//   RECEPTOR DE 5 CANAIS PARA CONTROLE REMOTO  INFRAVERMELHO (TV/VC)
//   COM OPO PARA MODO PULSO OU RETENO
//   MICROCONTROLADOR : PIC 12F675  - SENSOR TSOP 1550
//
//   Obs.Somente para protocolo 'SONY' de 12 bits.
//   timer 0 controla as piscada em modo de programao e o timer 1 conta
//   o tempo na recepo dos sinais infravermelhos oriundos do controle remoto.
//   Objetivo desta montagem:
//   Controlar cargas por meio de reles, acionados por controle remoto
//   com nmeros que podem ser escolhidos e programados pelo usurio.
//   Tem 5 canais diferentes e isolados entre s. Podem controlar cargas
//   AC (110 v) ou cargas DC ( 12 volts ,etc). O acionamento pode ser
//   programado por canal, em modo pulso ou reteno.
//
//==============================================================================
//
//   Como programar incialmente:
//   1) Aperte o boto 'Prog' por 2 segundos. Comear a piscar o led  do
//   canal 1 .
//   2) Aperte um boto do controle, da sua escolha, que deseje gravar na
//   Eeprom, para comandar o canal 1. Ao ser aceito, apagar o led do canal
//   1 e comear a pisca o led do canal 2.
//   3) Aperte um outro boto do controle,  para comandar o canal 2.
//   Ao ser aceito, apagar o led do canal 2 e piscara o led do canal 3.
//   4) Novamente, aperte um outro boto do controle,  para comandar o canal 3.
//   Ao ser aceito, apagar o led do canal 3 e piscar o led do canal 4.
//   5) Prossiga e aperte um outro boto do controle,  para comandar o canal 4.
//   Ao ser aceito, apagar o led do canal 4 e piscar o led do canal 5.
//   6) E finalmente, aperte um outro boto do controle, para comandar o canal 5.
//   Ao ser aceito, apagar o led do canal 5 e acender todos os leds de sada.
//   Isto indica que entramos na programao do modo de operao.
//
//   Siga os seguintes passos para escolher entre atuao 'Pulso' ou 'Reteno':
//   1) Aperte um dos botes do controle que foi gravado. A cada aperto, o led
//   do canal correspondente acender ou apagar. Deixando aceso ser modo 'pulso'
//   e deixando apagado ser modo 'reteno' para este canal.
//   2) Repita com os outros 4 botes que foram gravados.
//   3) Feito a gravao, aperte o boto 'Prog' por 2 segundos para sair do
//   modo de programao.
//
//   Uso desta montagem:
//   Ao apertar no controle remoto um boto previamente gravado, ele acionar
//   uma das sadas, em modo pulso ou reteno, dependendo de como foi programa-
//   do. Os reles devem ter condies de suportar as cargas de correntes em
//   seus contatos. Poder controlar 4 cargas diferentes, como ventiladores,
//   abajures, luminrias em geral, bem como outros aparelhos eletrodomsticos.
//   Use controle remoto com protocolo 'SONY 12 BITS' apenas. Outros protocolos
//   no iro  funcionar.
//==============================================================================
//   Obs. Este dispositivo  apenas didtico, sendo montado apenas em protoboard
//   e sendo sujeito a bugs no identificados.
//
//==============================================================================
//Descomente o abaixo SOMENTE se perdeu o byte de calibrao

    #define PERDI_BYTE_CALIBRACAO
//==============================================================================


#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(A)  //  sentido das portas feita pelo programador
#ifdef  PERDI_BYTE_CALIBRACAO  // calibrao do oscilador interno do pic
#ROM  0X3FF = {0X34fc}//mxima freq= 0x34fc ;mdia=0x3480; minima=0x3400
#endif
#ZERO_RAM

//==============================================================================
//   REGISTRADORES
//==============================================================================
#byte gpio   = 0x05
#byte tmr0   = 0x01
#byte tris   = 0x85
#byte t1con   =0x10
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte pir1    =0x0c
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte intcon  =0x8b
#byte status  =0x03
//==============================================================================
//   DEFINIO DE PINOS E SAIDAS
//==============================================================================
#bit  s1    = 0x5.4 //pino 3
#bit  s2    = 0x5.2 //pino 5
#bit  s3    = 0x5.1 //pino 6
#bit  s4    = 0x5.0 //pino 7
#bit  _prog = 0x5.3 //pino 4
#bit  s5    = 0x5.5 //pino 2
#bit  _in   = 0x5.3 //pino 4

#bit  cy     = 0x3.0 //carry bit
#bit  on_tmr1 = 0x10.0  //on/off tmr1
#bit  fov1    = 0x0c.0//flag over flow do tmr1
#bit  tmr1if  =  0x0c.0 //flag over flow do tmr1
#bit  tmr0if  = 0x8b.2//flag over tmr0

#define c1  pin_a4
#define c2  pin_a2
#define c3  pin_a1
#define c4  pin_a0
#define prog  pin_a3
#define c5   pin_a5
#define in  pin_a3

//#priority RA,timer0
//==============================================================================
//   CONSTANTES GERAIS
//==============================================================================
int const qbits = 12;  //quantidade de bits a receber 'sony'
//int const qbits = 32;  //quantidade de bits a receber 'nec'
int const TLIGADO=10; //30; //tempo mnimo para manter acionamento com boto apertado
int const   TINITPROG=150; // tempo mnimo para iniciar 'modo de programao'
int const  TIME_RX_ON=0x20;
//==============================================================================
//   VARIVEIS GLOBAIS
//==============================================================================

int qb,passo,ax1,cnt1,cnt2,cnt3,cnt4,cnt5,cntp1,grcnt;
int buffer[4];
short pisca,flag_prog,flag_prap,flag_rok;
short flag_m1, flag_m2, flag_m3,flag_m4,flag_m5;
short fs1,fs2,fs3,fs4,fs5,flag_stop;
//==============================================================================
//   SUBROTINAS E FUNES GERAIS
//==============================================================================


//==============================================================================
//   GRAVA EEPROM
//==============================================================================
void grava( int a,int b){
  if(!flag_stop){
  disable_interrupts(GLOBAL);
 write_eeprom(a,b);
 enable_interrupts(GLOBAL);
}
}




short mem(int a){

 return ((buffer[0]== read_eeprom(a))&&(buffer[1]== read_eeprom((a+1))));

}

//==============================================================================
//  grava memria
//==============================================================================

short rx_gr(void) {
 //if((flag_rok){
 if((flag_rok)&&(!flag_stop)){

         grava(passo,buffer[0]);
         grava(passo+1,buffer[1]);
         passo+=2;
         flag_stop=1;
         return 0;}
 else { return pisca;}
}



 //==============================================================================
//   INTERRUPO POR MUDANA DE ESTADO - RECEPO DO CONTROLE IR (PADRO 'SONY')
//==============================================================================
#int_RA
void rx_nec_mudanca_gp3() {
 int16 const  PULSO_2M4S=2000;
 static int16 ref,gt;
 static short ov1;


    gpio|=gpio; //ler port para conseguir zera gpif (DS41190C-page 21)
       if(!flag_rok){
           on_tmr1=0; ov1=tmr1if; gt=get_timer1(); set_timer1(0);
           tmr1if=0;  on_tmr1=1;//desliga timer1/pega valor/zera e religa

           cnt1=TLIGADO;
           cnt2=TLIGADO;
           cnt3=TLIGADO;
           cnt4=TLIGADO;
           cnt5=TLIGADO;

           if(input(in)){         //input='1'
           if((gt>PULSO_2M4S)||(ov1)){ qb=qbits;
            ref=gt; ref>>=2;ref+=ref>>1;    //obtem 900us
            buffer[0]=0;buffer[1]=0;
           }
               else {
                    shift_right(&buffer[0],2,(gt>ref));
                    if(--qb==0) { qb=qbits;
                                flag_rok=1;
                                  grcnt=TIME_RX_ON;
                                  
                                }
               }
            }
            else{                   //input='0'
                 if((gt>PULSO_2M4S)||(ov1)){ qb=qbits; }

            }

         }//if(!flag_rok)
}//void mudanca_gp3()



//==============================================================================

//=============================================================================
//   INTERRUPES DO TIMER 0
//==============================================================================

#int_timer0
void int_tmr0() {
static int t1_int;

      if(flag_prog) {
         if(flag_prap){ t1_int+=4;}//pisca led prog rapidamente
         else {++t1_int;} //pisca led prog lentamente

         if(t1_int>=10) {
            pisca=!pisca;
            t1_int=0;

            if(++ax1==0) --ax1;
            }
      }
      else{  pisca=0;}
}




//==============================================================================
//
//                PROGRAMA PRINCIPAL
//
//==============================================================================

void main() {

    //==============================================================================
//   INICIALISAO DE PORTAS E REGISTRADORES
//==============================================================================

   setup_adc_ports(no_analogs);  //somente pic12f675
   SETUP_ADC(ADC_OFF); //DESLIGA ADC
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_timer_1 ( T1_INTERNAL | T1_DIV_BY_1 );
   gpio=0;
   ax1=0;
   pisca=0;
   tris=0b001000; //tudo sada exceto gp3
   set_timer1(0); //zera tmr1
   tmr1if=0 ; //apaga flag over flow
   on_tmr1=1;//liga timer1
   setup_counters(RTCC_INTERNAL,RTCC_DIV_256);
   enable_interrupts(INT_RA3);//habilita interrupo por mudana de estado
   enable_interrupts(global);
   delay_ms(20);
   cntp1=TINITPROG;
   qb=qbits;//carrega quantidade de bits a receber
   passo=0;//  bit_set(t1con,0);
   flag_rok=0;
   flag_stop=0;
   grcnt=0;


//==============================================================================
//                         LOOP PRINCIPAL
//==============================================================================
   while(true){
      delay_ms(10);   //atraso temporizao correta do desligamento (modo Pulso)
    //==============================================================================
//   Desligamento de sadas em modo pulso
//==============================================================================


     if(!flag_prog){
     if(flag_m1){ if(--cnt1==0) s1=0;}//desliga saidas  em modo 'pulso'
     else { if(--cnt1==0) fs1=0;}


     if(flag_m2){ if(--cnt2==0) s2=0;}
     else { if(--cnt2==0) fs2=0;}

     if(flag_m3){ if(--cnt3==0) s3=0;}
     else { if(--cnt3==0) fs3=0;}

     if(flag_m4){ if(--cnt4==0) s4=0;}
     else { if(--cnt4==0) fs4=0;}

     if(flag_m5){ if(--cnt5==0) s5=0;}
     else { if(--cnt5==0) fs5=0;}

     }

//=============================================================================
//   TESTA BOTO 'PROG'  (INICIO/FIM DE MODO DE PROGRAMAO)
//==============================================================================



     if(!input(prog)) {
       if(--cntp1==0)  {
         cntp1=TINITPROG;
         if(flag_prog){ flag_prog=0; pisca=0;
         flag_prap=0;
         disable_interrupts(int_timer0);}
         else { flag_prog=1; passo=0; pisca=1; enable_interrupts(int_timer0);}
         gpio=0;
         }
     }
     else
     { cntp1=TINITPROG;}






 //==============================================================================
//   PROGRAMAR TECLAS DO  CONTROLE REMOTO NA EEPROM -MODO DE PROGRAMAO
//==============================================================================

  
     if(--grcnt==0xff){grcnt=0; flag_stop=0;};
     

     if(flag_prog){


       switch (passo){
       case 0:
         s1=rx_gr();
         break;
       case 2:
          s2=rx_gr();
         break;
       case 4:
          s3=rx_gr();
         break;
       case 6:
         s4=rx_gr();
         break;
       case 8:
         s5=rx_gr();
         break;

       case 9:
         if(!read_eeprom(10)){ flag_m1=0; s1=0;  }
            else{flag_m1=1; s1=1; }
         if(!read_eeprom(11)){ flag_m2=0; s2=0;  }
            else{flag_m2=1; s2=1; }
         if(!read_eeprom(12)){ flag_m3=0; s3=0;  }
            else{flag_m3=1; s3=1; }
         if(!read_eeprom(13)){ flag_m4=0; s4=0;  }
            else{flag_m4=1; s4=1; }
         if(!read_eeprom(14)){ flag_m5=0; s5=0;  }
            else{flag_m5=1; s5=1; }
        passo=10;


       case 10:

       if(flag_rok){
        if(mem(0)){
           if(read_eeprom(10)){ flag_m1=0; s1=0;  }
            else{flag_m1=1; s1=1; }
            grava(10,flag_m1);
            flag_stop=1;
            }
         else
        if(mem(2)){
          if(read_eeprom(11)){ flag_m2=0; s2=0;  }
            else{flag_m2=1; s2=1; }
            grava(11,flag_m2);
            flag_stop=1;
            }
         else

        if(mem(4)){
          if(read_eeprom(12)){ flag_m3=0; s3=0;  }
            else{flag_m3=1; s3=1; }
            grava(12,flag_m3);
            flag_stop=1;
            }
         else

         if(mem(6)){
           if(read_eeprom(13)){ flag_m4=0; s4=0;  }
            else{flag_m4=1; s4=1; }
            grava(13,flag_m4);
            flag_stop=1;
            }
         if(mem(8)){
            if(read_eeprom(14)){ flag_m5=0; s5=0;  }
            else{flag_m5=1; s5=1; }
            grava(14,flag_m5);
            flag_stop=1;
            }

        } //if(flag_rok)
      }//switch(passo)


       flag_rok=0;  //libera nova recepo



   }//if(flag_prog)


/*



 //==============================================================================
//   PROGRAMAR TECLAS DO  CONTROLE REMOTO NA EEPROM -MODO DE PROGRAMAO
//==============================================================================



     if(flag_prog){


       switch (passo){
       case 0:
         s1=pisca;
         if(flag_rok){
         grava(0,buffer[0]);
         grava(1,buffer[1]);
         s1=0;
         passo=2;}
         break;
       case 2:
         s2=pisca;
         if(flag_rok){
         grava(2,buffer[0]);
         grava(3,buffer[1]);
         passo=4;
         s2=0;}
         break;

       case 4:
         s3=pisca;
         if(flag_rok){
         grava(4,buffer[0]);
         grava(5,buffer[1]);
         passo=6;
         s3=0;}
         break;

       case 6:
         s4=pisca;
         if(flag_rok){
         grava(6,buffer[0]);
         grava(7,buffer[1]);
         s4=0;
         passo=8;}
         break;

       case 8:
         s5=pisca;
         if(flag_rok){
         grava(8,buffer[0]);
         grava(9,buffer[1]);
         flag_prap=1;//comea a piscar rpido o 'led prog'
         s5=0;
         passo=9;}
         break;





       case 9:
         if(!read_eeprom(10)){ flag_m1=0; s1=0;  }
            else{flag_m1=1; s1=1; }
         if(!read_eeprom(11)){ flag_m2=0; s2=0;  }
            else{flag_m2=1; s2=1; }
         if(!read_eeprom(12)){ flag_m3=0; s3=0;  }
            else{flag_m3=1; s3=1; }
         if(!read_eeprom(13)){ flag_m4=0; s4=0;  }
            else{flag_m4=1; s4=1; }
         if(!read_eeprom(14)){ flag_m5=0; s5=0;  }
            else{flag_m5=1; s5=1; }
        passo=10;


       case 10:

       if(flag_rok){
        if(mem(0)){
           if(read_eeprom(10)){ flag_m1=0; s1=0;  }
            else{flag_m1=1; s1=1; }
            grava(10,flag_m1);
            }
         else
        if(mem(2)){
          if(read_eeprom(11)){ flag_m2=0; s2=0;  }
            else{flag_m2=1; s2=1; }
            grava(11,flag_m2);
            }
         else

        if(mem(4)){
          if(read_eeprom(12)){ flag_m3=0; s3=0;  }
            else{flag_m3=1; s3=1; }
            grava(12,flag_m3);
            }
         else

         if(mem(6)){
           if(read_eeprom(13)){ flag_m4=0; s4=0;  }
            else{flag_m4=1; s4=1; }
            grava(13,flag_m4);
            }
         if(mem(8)){
            if(read_eeprom(14)){ flag_m5=0; s5=0;  }
            else{flag_m5=1; s5=1; }
            grava(14,flag_m5);
            }

        } //if(flag_rok)
      }//switch(passo)


       flag_rok=0;  //libera nova recepo



   }//if(flag_prog)







*/
 //=============================================================================
//   ACIONAR SADAS CONFORME PROGRAMAO
//==============================================================================


 if((!flag_prog)&&(flag_rok)){
  if(mem(0)){
   if(read_eeprom(10)){s1=1;flag_m1=1;cnt1=TLIGADO;}
   else { if (!fs1){s1=!s1; flag_m1=0;fs1=1;}
   }
   }
   else
 if(mem(2)){
    if(read_eeprom(11)){s2=1;flag_m2=1;cnt2=TLIGADO;}
  else { if (!fs2){s2=!s2; flag_m2=0; fs2=1;}
   }
   }

   else
 if(mem(4)){
   if(read_eeprom(12)){s3=1;flag_m3=1;cnt3=TLIGADO;}
   else { if (!fs3){s3=!s3; flag_m3=0; fs3=1;}
   }
   }
   else
 if(mem(6)){
   if(read_eeprom(13)){s4=1;flag_m4=1;cnt4=TLIGADO;}
   else { if (!fs4){s4=!s4; flag_m4=0; fs4=1;}
   }
   }
     else
 if(mem(8)){
   if(read_eeprom(14)){s5=1;flag_m5=1;cnt5=TLIGADO;}
   else { if (!fs5){s5=!s5; flag_m5=0; fs5=1;}
   }
   }

    flag_rok=0;
  }
 }//void while(true)
}//void main



